@@ -835,7 +835,7 @@ mod tests {
         assert_eq!(result.bindings.keys[0].key, "Q");
         assert_eq!(result.bindings.keys[0].with, "super");
         assert_eq!(result.bindings.keys[0].action.to_owned(), "Quit");
-        assert!(result.bindings.keys[0].text.to_owned().is_empty());
+        assert!(result.bindings.keys[0].esc.to_owned().is_empty());
     }
 
     #[test]
