@@ -23,6 +23,7 @@ use super::event::{
     ralt_pressed, scancode_to_physicalkey,
 };
 use super::window::WinitWindow;
+use super::window_delegate::WindowDelegate;
 use super::DEVICE_ID;
 use crate::dpi::{LogicalPosition, LogicalSize};
 use crate::event::{
@@ -134,6 +135,9 @@ pub struct ViewState {
     /// to the application, even during IME
     forward_key_to_app: Cell<bool>,
 
+    /// True if we're currently processing a key event
+    in_key_event: Cell<bool>,
+
     marked_text: RefCell<Retained<NSMutableAttributedString>>,
     accepts_first_mouse: bool,
 
@@ -335,12 +339,14 @@ declare_class!(
             trace_scope!("unmarkText");
             *self.ivars().marked_text.borrow_mut() = NSMutableAttributedString::new();
 
-            let input_context = self.inputContext().expect("input context");
-            input_context.discardMarkedText();
+            // Only discard marked text if we have an input context
+            if let Some(input_context) = self.inputContext() {
+                input_context.discardMarkedText();
+            }
 
             self.queue_event(WindowEvent::Ime(Ime::Preedit(String::new(), None)));
             if self.is_ime_enabled() {
-                // Leave the Preedit self.ivars()
+                // Leave the Preedit state
                 self.ivars().ime_state.set(ImeState::Ground);
             } else {
                 tracing::warn!("Expected to have IME enabled when receiving unmarkText");
@@ -376,10 +382,21 @@ declare_class!(
             _actual_range: *mut NSRange,
         ) -> NSRect {
             trace_scope!("firstRectForCharacterRange:actualRange:");
-            let rect = NSRect::new(
-                self.ivars().ime_position.get(),
-                self.ivars().ime_size.get()
-            );
+
+            let position = self.ivars().ime_position.get();
+            let size = self.ivars().ime_size.get();
+
+            // Validate coordinates before creating rect
+            if position.x.is_nan() || position.y.is_nan() || size.width.is_nan() || size.height.is_nan() {
+                tracing::warn!("Invalid IME coordinates in firstRectForCharacterRange");
+                // Return a default rect at origin with minimal size
+                let default_rect = NSRect::new(NSPoint::new(0.0, 0.0), NSSize::new(1.0, 1.0));
+                return self.window()
+                    .convertRectToScreen(self.convertRect_toView(default_rect, None));
+            }
+
+            let rect = NSRect::new(position, size);
+
             // Return value is expected to be in screen coordinates, so we need a conversion here
             self.window()
                 .convertRectToScreen(self.convertRect_toView(rect, None))
@@ -401,10 +418,24 @@ declare_class!(
             };
 
             let is_control = string.chars().next().is_some_and(|c| c.is_control());
+            let has_marked_text = unsafe { self.hasMarkedText() };
+            let is_in_key_event = self.ivars().in_key_event.get();
 
-            // Commit only if we have marked text.
-            if unsafe { self.hasMarkedText() } && self.is_ime_enabled() && !is_control {
-                self.queue_event(WindowEvent::Ime(Ime::Preedit(String::new(), None)));
+            if self.is_ime_enabled() {
+                if has_marked_text && !is_control {
+                    // Clear preedit and commit the text (normal IME flow)
+                    self.queue_event(WindowEvent::Ime(Ime::Preedit(String::new(), None)));
+                    self.queue_event(WindowEvent::Ime(Ime::Commit(string)));
+                    self.ivars().ime_state.set(ImeState::Committed);
+                } else if !is_control && !string.is_empty() && !is_in_key_event {
+                    // Direct input not from keyboard (e.g., emoji picker)
+                    self.queue_event(WindowEvent::Ime(Ime::Commit(string)));
+                    self.ivars().ime_state.set(ImeState::Committed);
+                }
+            } else if !is_control && !string.is_empty() && !is_in_key_event {
+                // IME is disabled but we got non-keyboard input (e.g., emoji picker)
+                // Temporarily enable IME for this input
+                self.queue_event(WindowEvent::Ime(Ime::Enabled));
                 self.queue_event(WindowEvent::Ime(Ime::Commit(string)));
                 self.ivars().ime_state.set(ImeState::Committed);
             }
@@ -436,6 +467,10 @@ declare_class!(
         #[method(keyDown:)]
         fn key_down(&self, event: &NSEvent) {
             trace_scope!("keyDown:");
+
+            // Set flag to indicate we're in a key event
+            self.ivars().in_key_event.set(true);
+
             {
                 let mut prev_input_source = self.ivars().input_source.borrow_mut();
                 let current_input_source = self.current_input_source();
@@ -490,12 +525,18 @@ declare_class!(
                     is_synthetic: false,
                 });
             }
+
+            // Clear the flag after processing
+            self.ivars().in_key_event.set(false);
         }
 
         #[method(keyUp:)]
         fn key_up(&self, event: &NSEvent) {
             trace_scope!("keyUp:");
 
+            // Set flag to indicate we're in a key event
+            self.ivars().in_key_event.set(true);
+
             let event = replace_event(event, self.option_as_alt());
             self.update_modifiers(&event, false);
 
@@ -510,6 +551,9 @@ declare_class!(
                     is_synthetic: false,
                 });
             }
+
+            // Clear the flag after processing
+            self.ivars().in_key_event.set(false);
         }
 
         #[method(flagsChanged:)]
@@ -798,6 +842,7 @@ impl WinitView {
             input_source: Default::default(),
             ime_allowed: Default::default(),
             forward_key_to_app: Default::default(),
+            in_key_event: Default::default(),
             marked_text: Default::default(),
             accepts_first_mouse,
             _ns_window: WeakId::new(&window.retain()),
@@ -900,10 +945,46 @@ impl WinitView {
     }
 
     pub(super) fn set_ime_cursor_area(&self, position: NSPoint, size: NSSize) {
+        // Validate coordinates to prevent invalid values that could cause IMK errors
+        if position.x.is_nan()
+            || position.y.is_nan()
+            || position.x < 0.0
+            || position.y < 0.0
+        {
+            tracing::warn!("Invalid IME cursor position: {:?}", position);
+            return;
+        }
+
+        if size.width.is_nan()
+            || size.height.is_nan()
+            || size.width <= 0.0
+            || size.height <= 0.0
+        {
+            tracing::warn!("Invalid IME cursor size: {:?}", size);
+            return;
+        }
+
+        // Only update if position or size has actually changed to avoid unnecessary IMK calls
+        let current_position = self.ivars().ime_position.get();
+        let current_size = self.ivars().ime_size.get();
+
+        if (current_position.x - position.x).abs() < 1.0
+            && (current_position.y - position.y).abs() < 1.0
+            && (current_size.width - size.width).abs() < 1.0
+            && (current_size.height - size.height).abs() < 1.0
+        {
+            return; // No significant change, skip update
+        }
+
         self.ivars().ime_position.set(position);
         self.ivars().ime_size.set(size);
-        let input_context = self.inputContext().expect("input context");
-        input_context.invalidateCharacterCoordinates();
+
+        // Only invalidate if we have an input context and IME is enabled
+        if let Some(input_context) = self.inputContext() {
+            if self.is_ime_enabled() {
+                input_context.invalidateCharacterCoordinates();
+            }
+        }
     }
 
     /// Reset modifiers and emit a synthetic ModifiersChanged event if deemed necessary.
@@ -1137,3 +1218,34 @@ fn replace_event(event: &NSEvent, option_as_alt: OptionAsAlt) -> Retained<NSEven
         event.copy()
     }
 }
+
+/// Get the window delegate from a view pointer
+/// This is used by the CVDisplayLink callback to access window state
+pub(crate) unsafe fn get_window_delegate(
+    view_ptr: *mut std::ffi::c_void,
+) -> Option<Retained<WindowDelegate>> {
+    if view_ptr.is_null() {
+        return None;
+    }
+
+    unsafe {
+        // Cast the view pointer back to WinitView
+        let view = view_ptr as *const WinitView;
+        let view = &*view;
+
+        // Get the window from the view
+        if let Some(window) = view.ivars()._ns_window.load() {
+            // Get the delegate from the window
+            if let Some(delegate) = window.delegate() {
+                // Cast the delegate back to WindowDelegate
+                // SAFETY: We know this is a WindowDelegate because we set it
+                let delegate_ptr = Retained::as_ptr(&delegate) as *mut WindowDelegate;
+                Retained::retain(delegate_ptr)
+            } else {
+                None
+            }
+        } else {
+            None
+        }
+    }
+}
