@@ -140,6 +140,12 @@ fn should_apps_use_dark_mode() -> bool {
         Lazy::new(|| unsafe {
             const UXTHEME_SHOULDAPPSUSEDARKMODE_ORDINAL: PCSTR = 132 as PCSTR;
 
+            // We won't try to do anything for windows versions < 17763
+            // (Windows 10 October 2018 update)
+            if !*DARK_MODE_SUPPORTED {
+                return None;
+            }
+
             let module = LoadLibraryA("uxtheme.dll\0".as_ptr());
 
             if module.is_null() {
